@@ -1,4 +1,4 @@
-/* $Id: proxycheck.c,v 1.16 2004/02/17 18:29:45 mjt Exp $
+/* $Id: proxycheck.c,v 1.23 2004/05/27 21:27:37 mjt Exp $
  * open proxy checker, main program.
  * Michael Tokarev  <mjt@corpit.ru>.
  * This code may be freely used and distributed according to
@@ -59,9 +59,9 @@ static int advanced;		/* use advanced protos as well */
 
 typedef struct {
   char *name;
-  void (*connh)(int fd, ev_t e, pxyconn_t *c);
+  void (*connh)(pxyconn_t *c, int e);
   int (*getdata)(pxyconn_t *c);
-  void (*exph)(int fd, ev_t e, pxyconn_t *c);
+  void (*exph)(pxyconn_t *c, int e);
   int (*init)(char *arg);
   char *arg;
   char *descr;
@@ -336,6 +336,25 @@ numprotoports(const pxyprotoport_t *pp) {
   return ppp - pp;
 }
 
+static void addent(char *p) {
+  char *n = p;
+  struct in_addr addr;
+  pxyhost_t *h;
+  if ((p = strchr(n, ':'))) *p = '\0';
+  if (!hostaddr(n, &addr))
+    usage("invalid IP address `%s'", n);
+  h = findhost(addr);
+  if (p) {
+    *p++ = ':';
+    if (*p)
+      h->pps = parseprotoport(h->pps, p);
+    else
+      h->nopen = 1; /* remember to add std pps */
+  }
+  else
+    h->nopen = 1; /* remember to add std pps */
+}
+
 static void
 init(int argc, char **argv) {
   int c;
@@ -347,7 +366,7 @@ init(int argc, char **argv) {
   pxyprotoport_t *pp;
   char *p;
   int npps;
-  int readin = 0;
+  const char *readin = NULL;
   int add_defaults = 0;
   char *check_arg = NULL;
 
@@ -413,7 +432,7 @@ init(int argc, char **argv) {
       break;
 
     case 'i':
-      readin = 1;
+      readin = optarg;
       break;
 
     case 'n':
@@ -490,22 +509,28 @@ init(int argc, char **argv) {
   if (!check)
     usage("no action (-c) specified");
 
-  while(*argv) {
-    char *n = *argv++;
-    struct in_addr addr;
-    if ((p = strchr(n, ':'))) *p = '\0';
-    if (!hostaddr(n, &addr))
-      usage("invalid IP address `%s'", n);
-    h = findhost(addr);
-    if (p) {
-      *p++ = ':';
-      if (*p)
-        h->pps = parseprotoport(h->pps, p);
-      else
-        h->nopen = 1; /* remember to add std pps */
+  while(*argv)
+    addent(*argv++);
+  if (readin) {
+    FILE *f;
+    char buf[8192];
+    if (readin[0] == '-' && readin[1] == '\0')
+      f = stdin;
+    else if ((f = fopen(readin, "r")) == NULL)
+      err(errno, "unable to open %s", readin);
+    while(fgets(buf, sizeof(buf), f)) {
+      char *p = buf;
+      char *e;
+      while(*p == ' ' || *p == '\t')
+       ++p;
+      if (*p == '#' || *p == '\n' || !*p)
+        continue;
+      if ((e = strchr(p, '\n')) != NULL)
+        *e = '\0';
+      addent(p);
     }
-    else
-      h->nopen = 1; /* remember to add std pps */
+    if (f != stdin)
+      fclose(f);
   }
 
   if (getuid() == 0 || geteuid() == 0)
@@ -596,8 +621,8 @@ nextconn(void) {
     return 0;
 
   fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
-  if (fd < 0 || fd >= FD_SETSIZE) {
-    if (fd < 0 && errno != EMFILE)
+  if (fd < 0) {
+    if (errno != EMFILE)
       err(errno, "unable to create socket");
     if (!maxconn || maxconn > nactive) {
       pxyinfo(NULL, 1, "limiting max number of connections to %d",
@@ -635,7 +660,7 @@ nextconn(void) {
   /* from now on, we should expect to be called recursively:
    * do not touch chost and pp anymore */
 
-  if (!pxystart(fd, c))
+  if (!pxystart(c, fd))
     err(errno, "unable to enqueue testing request");
 
   sin.sin_family = AF_INET;
@@ -650,13 +675,14 @@ nextconn(void) {
 
 static void
 freeconn(pxyconn_t *c) {
+  int fd = c->fd;
   pxyhost_t *h = (pxyhost_t *)c->data;
   nread += c->nread;
   --h->nactive;
   --nactive;
   pxyinfo(c, 5, "done, nactive=%d ntodo=%d", nactive, ntodo);
-  close(c->fd);
   pxyfree(c);
+  close(fd);
   if (stopfound && h->nopen && h->cpp->port) {
     /*XXX abort active connections: need a list of
      * all connections for this host, and a state of
@@ -709,8 +735,8 @@ isopen(pxyconn_t *c, int is_open, const char *info) {
 
 void pxyaction(pxyconn_t *c, int result) {
   if (!result) {
-    if (pxyreqiot(c, EV_READ, check->connh, 0))
-      check->connh(c->fd, EV_READ, c);
+    if (pxyreqiot(c, EV_IN, check->connh, 0, NULL))
+      check->connh(c, EV_IN);
   }
   else {
     if (result < 0)
@@ -727,8 +753,8 @@ void pxyaction(pxyconn_t *c, int result) {
 }
 
 void pxycheckdata(pxyconn_t *c) {
-  if (pxyreqio(c, EV_READ, check->exph))
-      check->exph(c->fd, EV_READ, c);
+  if (pxyreqio(c, EV_IN, check->exph))
+    check->exph(c, EV_IN);
 }
 
 int pxygetdata(pxyconn_t *c) {
@@ -738,14 +764,16 @@ int main(int argc, char **argv) {
   time_t start;
 
   init(argc, argv);
+  if (ev_init(0, EV_ADVANCED|EV_SELECT) != 0)
+    err(errno, "ev_init");
 
   start = time(NULL);
 
   while(nextconn())
     ;
 
-  while(ev_hasioevents())
-    if (ev_wait(-1) < 0 && errno != EINTR)
+  while(nactive)
+    if (ev_wait(0, -1) < 0 && errno != EINTR)
       err(errno, "ev_wait");
 
   pxyinfo(NULL, 1, "NumOpen=%d(%d) NRead=%ld Time=%d",
@@ -781,11 +809,10 @@ static int expectd(pxyconn_t *c) {
 }
 
 static void
-expectw(int UNUSED fd, ev_t e, pxyconn_t *c) {
+expectw(pxyconn_t *c, int UNUSED e) {
   int l;
   char *p;
-  if (!pxynoerr(e, c)) return;
-  else if (pxyreadnext(c, 1, &l, 3) <= 0)
+  if (pxyreadnext(c, 1, &l, 3) <= 0)
     ;
   else if (c->proto->check && c->proto->check(c, pxybuf, l)) return;
   else if ((p = (char*)memmem(pxybuf, l, expstr, explen)) != NULL)
@@ -796,10 +823,9 @@ expectw(int UNUSED fd, ev_t e, pxyconn_t *c) {
 }
 
 static void
-expecth(int UNUSED fd, ev_t e, pxyconn_t *c) {
-  if (!pxynoerr(e, c)) return;
-  else if (pxywrite(c, pxybuf, expectd(c), 3))
-    pxyreqio(c, EV_READ, expectw);
+expecth(pxyconn_t *c, int UNUSED e) {
+  if (pxywrite(c, pxybuf, expectd(c), 3))
+    pxyreqio(c, EV_IN, expectw);
 }
 
 static int expecti(char *arg) {
@@ -869,10 +895,9 @@ dsblo(pxyconn_t *c, char *line) {
 }
 
 static void
-dsble(int UNUSED fd, ev_t e, pxyconn_t *c) {
+dsble(pxyconn_t *c, int UNUSED e) {
   int l;
   char *s;
-  if (!pxynoerr(e, c)) return;
   /* state:
    * 0 - connected, 2xx helo
    * 1 - 2xx mail from
@@ -898,7 +923,7 @@ dsble(int UNUSED fd, ev_t e, pxyconn_t *c) {
 }
 
 static void
-dsblh(int UNUSED fd, ev_t e, pxyconn_t *c) {
+dsblh(pxyconn_t *c, int UNUSED e) {
   /* state:
    * 0 - connected, send helo
    * 1 - wait for initial 2xx reply, send MAIL FROM
@@ -909,7 +934,6 @@ dsblh(int UNUSED fd, ev_t e, pxyconn_t *c) {
    */
   int l;
   char *p;
-  if (!pxynoerr(e, c)) return;
 
   switch(c->appstate) {
   case 0: /* connected, send helo */
@@ -928,7 +952,7 @@ dsblh(int UNUSED fd, ev_t e, pxyconn_t *c) {
           sprintf(pxybuf, "DATA\r\n");
       if (pxysave(c, pxybuf, l, 10) &&
           pxywrite(c, pxybuf, l, 3) &&
-          pxyrenew(c, pxytimeout/2))
+          pxyrenew(c, pxytimeout/2, NULL))
         c->appstate += 1;
     }
     else if (c->appstate != 1 ||
@@ -943,7 +967,7 @@ dsblh(int UNUSED fd, ev_t e, pxyconn_t *c) {
       if (verbose < 6) pxyinfo(c, 2, "sending message");
       if (pxywrite(c, pxybuf, dsblmsg(pxybuf, c), 6) &&
           pxywrite(c, "QUIT\r\n", 6, 3) &&
-          pxysave(c, NULL, 0, 0) && pxyrenew(c, 0))
+          pxysave(c, NULL, 0, 0) && pxyrenew(c, 0, NULL))
         c->appstate = 5;
     }
     else
@@ -965,6 +989,10 @@ static char *egetenv(const char *name, char *def) {
   return v ? v : def;
 }
 
+static void dsbl_cookie_alarm(int UNUSED sig) {
+  err(0, "unable to obtain cookie: timeout");
+}
+
 static int
 dsbli(char *arg) {
   if (!dstspec)
@@ -988,6 +1016,8 @@ dsbli(char *arg) {
     sin.sin_port = htons(DSBL_COOKIE_PORT);
     if (!hostaddr(chost, &sin.sin_addr))
       err(0, "unknown DSBL cookie host %s", chost);
+    signal(SIGALRM, dsbl_cookie_alarm);
+    alarm(3*60);
     if ((fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 ||
         connect(fd, (struct sockaddr*)&sin, sizeof(sin)) < 0)
       err(errno, "unable to connect to cookie server");
@@ -995,6 +1025,7 @@ dsbli(char *arg) {
     if (send(fd, pxybuf, l, 0) < 0 ||
         recv(fd, dsblcookie, DSBL_COOKIE_LEN, 0) != DSBL_COOKIE_LEN)
       err(errno, "unable to obtain cookie");
+    alarm(0);
     close(fd);
     dsblcookie[DSBL_COOKIE_LEN] = 0;
   }
