@@ -1,4 +1,4 @@
-/* $Id: pxy.c,v 1.16 2004/02/17 18:30:42 mjt Exp $
+/* $Id: pxy.c,v 1.19 2004/05/27 14:18:43 mjt Exp $
  * open proxy checker, proxy protocol routines.
  * Michael Tokarev  <mjt@corpit.ru>.
  * This code may be freely used and distributed according to
@@ -20,13 +20,16 @@
 int pxytimeout = 30000; /* 30 sec */
 char pxybuf[8193];
 
-static void timedout(int UNUSED fd, ev_t UNUSED e, void *ctx) {
+static void
+timedout(pxyconn_t *c) {
   errno = ETIMEDOUT;
-  ev_iowakeup((int)ctx, EV_ERRNO);
+  pxyaction(c, -1);
 }
 
-int pxyrenew(pxyconn_t *c, int tmo) {
-  if (ev_settimer(tmo ? tmo : pxytimeout, timedout, (void*)c->fd) == 0)
+int pxyrenew(pxyconn_t *c, int tmo, void (*tmfn)(pxyconn_t *)) {
+  if (!tmfn) tmfn = timedout;
+  ev_tm_del(0, &c->timer);
+  if (ev_tm_add(0, &c->timer, tmo ? tmo : pxytimeout, (ev_tm_cbck_f*)tmfn, c) != NULL)
     return 1;
   else {
     pxyaction(c, -1);
@@ -34,17 +37,6 @@ int pxyrenew(pxyconn_t *c, int tmo) {
   }
 }
 
-int pxynoerr(ev_t e, pxyconn_t *c) {
-  if (e & EV_RDWR)
-    return 1;
-  /*
-  if (errno != ETIMEDOUT)
-    pxyinfo(c, 1, "error: %s", strerror(errno));
-  */
-  pxyaction(c, -1);
-  return 0;
-}
-
 pxyconn_t *pxynew() {
   pxyconn_t *c = (pxyconn_t *)malloc(sizeof(pxyconn_t));
   if (c)
@@ -53,10 +45,9 @@ pxyconn_t *pxynew() {
 }
 
 void pxyfree(pxyconn_t *c) {
-  if (c->fd >= 0) {
-    ev_settimer(-1, timedout, (void*)c->fd);
-    ev_reqio(c->fd, 0, NULL, NULL);
-  }
+  ev_tm_del(0, &c->timer);
+  if (c->fd >= 0)
+    ev_io_del(0, c->fd);
   if (c->detail)
     free(c->detail);
   if (c->buf) {
@@ -67,8 +58,8 @@ void pxyfree(pxyconn_t *c) {
 }
 
 int
-pxyreqio(pxyconn_t *c, ev_t e, void(*fn)(int,ev_t,pxyconn_t*)) {
-  if (ev_reqio(c->fd, e, (void(*)(int,ev_t,void*))fn, c) == 0)
+pxyreqio(pxyconn_t *c, int e, void(*iofn)(pxyconn_t*,int)) {
+  if (ev_io_mod(0, c->fd, e, (ev_io_cbck_f*)iofn, c) == 0)
     return 1;
   else {
     pxyaction(c, -1);
@@ -77,8 +68,9 @@ pxyreqio(pxyconn_t *c, ev_t e, void(*fn)(int,ev_t,pxyconn_t*)) {
 }
 
 int
-pxyreqiot(pxyconn_t *c, ev_t e, void(*fn)(int,ev_t,pxyconn_t*), int tmo) {
-  return pxyreqio(c, e, fn) && pxyrenew(c, tmo);
+pxyreqiot(pxyconn_t *c, int e, void(*iofn)(pxyconn_t*,int),
+          int tmo, void(*tmfn)(pxyconn_t*)) {
+  return pxyreqio(c, e, iofn) && pxyrenew(c, tmo, tmfn);
 }
 
 int pxywrite(pxyconn_t *c, const char *buf, int len, int level) {
@@ -102,11 +94,14 @@ int pxyprintf(pxyconn_t *c, int loglevel, const char *fmt, ...) {
   return pxywrite(c, pxybuf, l, loglevel);
 }
 
-int pxystart(int fd, pxyconn_t *c) {
+int pxystart(pxyconn_t *c, int fd) {
   c->fd = fd;
-  return
-    pxyreqio(c, EV_WRITE, c->proto->handler) &&
-    pxyrenew(c, pxytimeout/2);
+  if (ev_io_add(0, fd, EV_OUT, (ev_io_cbck_f*)c->proto->handler, c) != 0) {
+    pxyaction(c, -1);
+    return 0;
+  }
+  else
+    return pxyrenew(c, pxytimeout/2, NULL);
 }
 
 int pxyread(pxyconn_t *c, char *buf, int l, int minlen, int level) {
@@ -273,19 +268,17 @@ static int hcc(pxyconn_t *c, char *buf, int l) {
   }
 }
 
-static void
-hch(int UNUSED fd, ev_t e, pxyconn_t *c) {
+static void hch(pxyconn_t *c, int UNUSED e) {
   /* HTTP CONNECT.
    * state:
    *  1 - request sent, want reply
    */
-  if (!pxynoerr(e, c)) return;
   if (c->pxystate == 0) {
     /* some proxies requires \r\n in separate packet - doh */
     if (pxyprintf(c, 3, "CONNECT %s:%d HTTP/1.0\r\n",
                   inet_ntoa(c->dstaddr), c->dstport) &&
         pxywrite(c, "\r\n", 2, 3) &&
-        pxyreqiot(c, EV_READ, hch, 0))
+        pxyreqiot(c, EV_IN, hch, 0, NULL))
       c->pxystate = 1;
   }
   else
@@ -311,9 +304,7 @@ hxc(pxyconn_t *c, char *buf, int l) {
   }
 }
 
-static void
-hxh(int UNUSED fd, ev_t e, pxyconn_t *c) {
-  if (!pxynoerr(e, c)) return;
+static void hxh(pxyconn_t *c, int UNUSED e) {
   if (!c->pxystate) {
     int dl = pxygetdata(c);
     char *b = pxybuf + dl + 1;
@@ -328,28 +319,26 @@ hxh(int UNUSED fd, ev_t e, pxyconn_t *c) {
       c->pxystate = 1;
       pxyinfo(c, 3, "sending data");
       if (pxywrite(c, pxybuf, dl, 6))
-        pxyreqiot(c, EV_READ, hxh, 0);
+        pxyreqiot(c, EV_IN, hxh, 0, NULL);
     }
   }
   else
     pxycheckdata(c);
 }
 
-static void
-s5h(int UNUSED fd, ev_t e, pxyconn_t *c) {
+static void s5h(pxyconn_t *c, int UNUSED e) {
   /* SOCKS5
    * state:
    *  1 - auth request sent, want reply
    *  2 - connect request sent, want reply
    */
   int l;
-  if (!pxynoerr(e, c)) return;
   if (!c->pxystate) {
     pxybuf[0] = 5; /* socks version */
     pxybuf[1] = 1; /* nmethods */
     pxybuf[2] = 0; /* no auth reqd */
     if (pxywrite(c, pxybuf, 3, 4) &&
-        pxyreqiot(c, EV_READ, s5h, pxytimeout / 2))
+        pxyreqiot(c, EV_IN, s5h, pxytimeout / 2, NULL))
       c->pxystate = 1;
   }
   else if (c->pxystate == 1) {
@@ -377,7 +366,7 @@ s5h(int UNUSED fd, ev_t e, pxyconn_t *c) {
       pxybuf[3] = 1; /* ipv4 addrtype */
       memcpy(pxybuf+4, &c->dstaddr.s_addr, 4);
       memcpy(pxybuf+8, &p, 2);
-      if (pxywrite(c, pxybuf, 10, 4) && pxyrenew(c, 0))
+      if (pxywrite(c, pxybuf, 10, 4) && pxyrenew(c, 0, NULL))
         c->pxystate = 2;
     }
   }
@@ -407,14 +396,12 @@ s5h(int UNUSED fd, ev_t e, pxyconn_t *c) {
   }
 }
 
-static void
-s4h(int UNUSED fd, ev_t e, pxyconn_t *c) {
+static void s4h(pxyconn_t *c, int UNUSED e) {
   /* SOCKS4.
    * state:
    *  1 - connect request sent, want reply
    */
   int l;
-  if (!pxynoerr(e, c)) return;
   if (!c->pxystate) {
     ipport_t p = htons(c->dstport);
     pxybuf[0] = 4; /* socks version */
@@ -422,7 +409,7 @@ s4h(int UNUSED fd, ev_t e, pxyconn_t *c) {
     memcpy(pxybuf+2, &p, 2);
     memcpy(pxybuf+4, &c->dstaddr.s_addr, 4);
     pxybuf[8] = 0; /* empty/null username */
-    if (pxywrite(c, pxybuf, 9, 4) && pxyreqiot(c, EV_READ, s4h, 0))
+    if (pxywrite(c, pxybuf, 9, 4) && pxyreqiot(c, EV_IN, s4h, 0, NULL))
       c->pxystate = 1;
   }
   else if ((l = pxyread(c, pxybuf, 8, 8, 4)) <= 0)
@@ -478,8 +465,13 @@ wgc(pxyconn_t *c, char *buf, int l) {
   return 1;
 }
 
-static void
-wgh(int UNUSED fd, ev_t e, pxyconn_t *c) {
+static void wgh(pxyconn_t *c, int e);
+
+static void wgh_tmo(pxyconn_t *c) {
+  wgh(c, -1);
+}
+
+static void wgh(pxyconn_t *c, int e) {
   /* WINGATE/telnet.
    * state:
    *  1 - wait for a prompt, interpret Username:/Login: and similar;
@@ -489,26 +481,23 @@ wgh(int UNUSED fd, ev_t e, pxyconn_t *c) {
    */
   int l;
   if (!c->pxystate) {
-    if (!pxynoerr(e, c)) return;
     if (pxyreadnext(c, 0, &l, 3) < 0)
       return;
     c->pxystate = 1;
-    if (!pxyreqiot(c, EV_READ, wgh, pxytimeout / 8) ||
+    if (!pxyreqiot(c, EV_IN, wgh, pxytimeout / 8, wgh_tmo) ||
         !l || !pxysave(c, pxybuf, l, 0))
       return;
   }
   if (c->pxystate == 1) {
-    if (e == EV_ERRNO && errno == ETIMEDOUT) {
+    if (e < 0) {
       /*XX try to recognize whenever to use telnet or "plain" wingate */
       l = sprintf(pxybuf, "%s:%d\r\n", inet_ntoa(c->dstaddr), c->dstport);
       if (c->pxyport == 23)
         l += sprintf(pxybuf + l, "telnet %s %d\r\n",
                      inet_ntoa(c->dstaddr), c->dstport);
-      if (pxywrite(c, pxybuf, l, 3) && pxyrenew(c, 0))
+      if (pxywrite(c, pxybuf, l, 3) && pxyrenew(c, 0, NULL))
         c->pxystate = 2;
     }
-    else if (!pxynoerr(e, c))
-      return;
     else if (pxyreadnext(c, 0, &l, 3) < 0 || wgc(c, pxybuf, l))
       return;
     else {
@@ -561,17 +550,16 @@ wgh(int UNUSED fd, ev_t e, pxyconn_t *c) {
       default:
         return;
       }
-      if (pxysave(c, pxybuf, l, 0) && pxyrenew(c, 0))
+      if (pxysave(c, pxybuf, l, 0) && pxyrenew(c, 0, NULL))
         c->pxystate = 2;
       c->detail = strdup(d);
     }
   }
-  else if (pxynoerr(e, c))
+  else
     pxyaction(c, 0);
 }
 
-static void
-fuh(int UNUSED fd, ev_t e, pxyconn_t *c) {
+static void fuh(pxyconn_t *c, int UNUSED e) {
   /* FTP proxy.  Wait for initial FTP greething and
    * issue a command:
    *   USER dummy@targethost:targetport\r\n
@@ -582,12 +570,11 @@ fuh(int UNUSED fd, ev_t e, pxyconn_t *c) {
    * dependant on first line from client (e.g. http).
    */
   int l;
-  if (!pxynoerr(e, c)) return;
   if (!c->pxystate) {
     if (pxyreadnext(c, 0, &l, 3) < 0) /* check if established */
       return;
     c->pxystate = 1;
-    if (!pxyreqiot(c, EV_READ, fuh, pxytimeout / 2) ||
+    if (!pxyreqiot(c, EV_IN, fuh, pxytimeout / 2, NULL) ||
         !l || !pxysave(c, pxybuf, l, 0))
       return;
   }
